@@ -17,6 +17,7 @@
 #include <asm/uaccess.h>
 #include <linux/freezer.h>
 #include <linux/kthread.h>
+#include <linux/dma-mapping.h>
 
 #include <linux/fs.h>
 #include <linux/cdev.h>
@@ -55,6 +56,7 @@ MODULE_DEVICE_TABLE(pci, pt1_pci_tbl);
 #define		DEV_NAME	"pt1video"
 
 #define		MAX_READ_BLOCK	4			// 1٤ɤ߽ФDMAХåե
+#define		MAX_PCI_DEVICE		64		// 64
 #define		DMA_SIZE	4096			// DMAХåե
 #define		DMA_RING_SIZE	64			// RING
 #define		DMA_RING_MAX	511			// 1RINGˤĵͤ뤫(1023NG511ޤ)
@@ -77,6 +79,7 @@ typedef	struct	_pt1_device{
 	void			*dmaptr[DMA_RING_SIZE] ;
 	struct	task_struct	*kthread;
 	dev_t			dev ;
+	__u32			base_minor ;
 	struct	cdev	cdev[MAX_CHANNEL];
 	wait_queue_head_t	dma_wait_q ;// for poll on reading
 	DMA_CONTROL		dmactl[DMA_RING_SIZE];
@@ -101,6 +104,7 @@ struct	_PT1_CHANNEL{
 	__u32			overflow ;		// Сե顼ȯ
 	__u32			counetererr ;	// ž󥿣顼
 	__u32			transerr ;		// ž顼
+	__u32			minor ;			// ޥʡֹ
 	__u8			*buf;			// CH̼
 	__u8			req_dma ;		// 줿ͥ
 	PT1_DEVICE		*ptr ;			// ̾
@@ -113,7 +117,7 @@ int		real_chanel[MAX_CHANNEL] = {0, 2, 1, 3};
 int		channeltype[MAX_CHANNEL] = {CHANNEL_TYPE_ISDB_S, CHANNEL_TYPE_ISDB_S,
 									CHANNEL_TYPE_ISDB_T, CHANNEL_TYPE_ISDB_T};
 
-static	PT1_DEVICE	*device = NULL;
+static	PT1_DEVICE	*device[MAX_PCI_DEVICE];
 
 #define		PT1MAJOR	251
 #define		DRIVERNAME	"pt1video"
@@ -123,7 +127,20 @@ static	void	reset_dma(PT1_DEVICE *dev_conf)
 
 	int		lp ;
 	__u32	addr ;
+	int		ring_pos = 0;
+	int		data_pos = 0 ;
+	__u32	*dataptr ;
 
+	// ǡ
+	for(ring_pos = 0 ; ring_pos < DMA_RING_SIZE ; ring_pos++){
+		for(data_pos = 0 ; data_pos < DMA_RING_MAX ; data_pos++){
+			dataptr = dev_conf->dmactl[ring_pos].data[data_pos];
+			// ǡꡩ
+			if(dataptr[(DMA_SIZE / sizeof(__u32)) - 2] == 0){
+				break ;
+			}
+		}
+	}
 	// ž󥿤ꥻå
 	writel(0x00000010, dev_conf->regs);
 	// ž󥿤򥤥󥯥
@@ -131,7 +148,7 @@ static	void	reset_dma(PT1_DEVICE *dev_conf)
 		writel(0x00000020, dev_conf->regs);
 	}
 
-	addr = (int)dev_conf->dmaptr[0] ;
+	addr = (int)dev_conf->ring_dma[0] ;
 	addr >>= 12 ;
 	// DMAХåե
 	writel(addr, dev_conf->regs + DMA_ADDR);
@@ -158,8 +175,8 @@ static	int		pt1_thread(void *data)
 	}micro;
 
 	set_freezable();
-	printk(KERN_INFO "pt1_thread run\n");
 	reset_dma(dev_conf);
+	printk(KERN_INFO "pt1_thread run\n");
 
 	for(;;){
 		if(kthread_should_stop()){
@@ -174,6 +191,7 @@ static	int		pt1_thread(void *data)
 			}
 			micro.val = *dataptr ;
 			curdataptr = dataptr ;
+			data_pos += 1 ;
 			for(lp = 0 ; lp < (DMA_SIZE / sizeof(__u32)) ; lp++, dataptr++){
 				micro.val = *dataptr ;
 				chno = real_chanel[(((micro.packet.head >> 5) & 0x07) - 1)];
@@ -190,36 +208,41 @@ static	int		pt1_thread(void *data)
 					if((val & BIT_INITIATOR_WARNING)){
 						channel->transerr += 1 ;
 					}
+					// Ƭ
 					reset_dma(dev_conf);
+					ring_pos = data_pos = 0 ;
+					break ;
 				}
 				// ̤ѥͥϼΤƤ
 				if(channel->valid == FALSE){
 					continue ;
 				}
 				spin_lock(&channel->lock);
-				// Ȥꤢ顼åʤǥǡΤߥԡ
-				// դ줿Ԥ
-				if(channel->size < (channel->maxsize - 4)){
-					for(lp2 = 2 ; lp2 >= 0 ; lp2--){
-						channel->buf[channel->size] = micro.packet.data[lp2];
-						channel->size += 1 ;
+				// դ줿ɤ߽ФޤԤ
+				while(1){
+					if(channel->size >= (channel->maxsize - 4)){
+						// ͥDMAɤߤԤˤ
+						wake_up(&channel->wait_q);
+						channel->req_dma = TRUE ;
+						spin_unlock(&channel->lock);
+						// ˻֤Ϥ
+						wait_event_timeout(dev_conf->dma_wait_q, (channel->req_dma == FALSE),
+											msecs_to_jiffies(500));
+						spin_lock(&channel->lock);
+						channel->drop += 1 ;
+					}else{
+						break ;
 					}
-				}else{
-					// ͥDMAɤߤԤˤ
-					wake_up(&channel->wait_q);
-					channel->req_dma = TRUE ;
-					spin_unlock(&channel->lock);
-					// ˻֤Ϥ
-					wait_event_timeout(dev_conf->dma_wait_q, (channel->req_dma == FALSE),
-										msecs_to_jiffies(500));
-					spin_lock(&channel->lock);
-					channel->drop += 1 ;
+				}
+				// ǡԡ
+				for(lp2 = 2 ; lp2 >= 0 ; lp2--){
+					channel->buf[channel->size] = micro.packet.data[lp2];
+					channel->size += 1 ;
 				}
 				spin_unlock(&channel->lock);
 			}
 			curdataptr[(DMA_SIZE / sizeof(__u32)) - 2] = 0;
 
-			data_pos += 1 ;
 			if(data_pos >= DMA_RING_MAX){
 				data_pos = 0;
 				ring_pos += 1 ;
@@ -246,35 +269,52 @@ static int pt1_open(struct inode *inode, struct file *file)
 {
 
 	int		minor = iminor(inode);
+	int		lp ;
+	int		lp2 ;
 	PT1_CHANNEL	*channel ;
 
-	if(device == NULL){
-		return -EIO ;
+	for(lp = 0 ; lp < MAX_PCI_DEVICE ; lp++){
+		if(device[lp] == NULL){
+			return -EIO ;
+		}
+		printk(KERN_INFO "(%d)base_minor=%d: MAX=%d(%d)\n",
+				lp, device[lp]->base_minor,
+				(device[lp]->base_minor + MAX_CHANNEL), minor);
+		if((device[lp]->base_minor <= minor) &&
+			((device[lp]->base_minor + MAX_CHANNEL) > minor)){
+			spin_lock(&device[lp]->lock);
+			for(lp2 = 0 ; lp2 < MAX_CHANNEL ; lp2++){
+				channel = device[lp]->channel[lp2] ;
+				printk(KERN_INFO "Minor(%d:%d)\n", channel->minor, minor);
+				if(channel->minor == minor){
+					if(channel->valid == TRUE){
+						spin_unlock(&device[lp]->lock);
+						return -EIO ;
+					}
+					channel->drop  = 0 ;
+					channel->valid = TRUE ;
+					channel->overflow = 0 ;
+					channel->counetererr = 0 ;
+					channel->transerr = 0 ;
+					file->private_data = channel;
+					spin_lock(&channel->lock);
+					// ǡ
+					channel->size = 0 ;
+					spin_unlock(&channel->lock);
+					spin_unlock(&device[lp]->lock);
+					return 0 ;
+				}
+			}
+		}
 	}
-	spin_lock(&device->lock);
-	channel = device->channel[minor] ;
-	if(channel->valid == TRUE){
-		spin_unlock(&device->lock);
-		return -EIO ;
-	}
-	channel->drop  = 0 ;
-	channel->valid = TRUE ;
-	channel->overflow = 0 ;
-	channel->counetererr = 0 ;
-	channel->transerr = 0 ;
-	file->private_data = channel;
-	spin_lock(&channel->lock);
-	// ǡ
-	channel->size = 0 ;
-	spin_unlock(&channel->lock);
-	spin_unlock(&device->lock);
-	return 0;
+	return -EIO;
 }
 static int pt1_release(struct inode *inode, struct file *file)
 {
+	int		minor = iminor(inode);
 	PT1_CHANNEL	*channel = file->private_data;
 
-	spin_lock(&device->lock);
+	spin_lock(&channel->ptr->lock);
 	SetStream(channel->ptr->regs, channel->channel, FALSE);
 	channel->valid = FALSE ;
 	printk(KERN_INFO "(%d)Drop=%08d:%08d:%08d:%08d\n", iminor(inode), channel->drop,
@@ -288,7 +328,7 @@ static int pt1_release(struct inode *inode, struct file *file)
 		channel->req_dma = FALSE ;
 		wake_up(&channel->ptr->dma_wait_q);
 	}
-	spin_unlock(&device->lock);
+	spin_unlock(&channel->ptr->lock);
 	return 0;
 }
 
@@ -336,7 +376,6 @@ static	int		SetFreq(PT1_CHANNEL *channel, FREQUENCY *freq)
 				ISDB_S_TMCC		tmcc ;
 				int				lp ;
 
-				printk(KERN_INFO "TUNE(%d:%x)\n", freq->frequencyno, freq->slot);
 				if(bs_tune(channel->ptr->regs,
 						channel->ptr->lock,
 						channel->address,
@@ -344,9 +383,26 @@ static	int		SetFreq(PT1_CHANNEL *channel, FREQUENCY *freq)
 						&tmcc) < 0){
 					return -EIO ;
 				}
+				printk(KERN_INFO "cn = (%x:%x)\n", (tmcc.cn[0] & 0xFF),
+						(tmcc.cn[1] & 0xFF));
+				printk(KERN_INFO "agc = (%x)\n", (tmcc.agc & 0xFF));
+
+#if 0
+				printk(KERN_INFO "clockmargin = (%x)\n", (tmcc.clockmargin & 0xFF));
+				printk(KERN_INFO "carriermargin  = (%x)\n", (tmcc.carriermargin & 0xFF));
+
 				for(lp = 0 ; lp < MAX_BS_TS_ID ; lp++){
+					if(tmcc.ts_id[lp].ts_id == 0xFFFF){
+						continue ;
+					}
 					printk(KERN_INFO "Slot(%d:%x)\n", lp, tmcc.ts_id[lp].ts_id);
+					printk(KERN_INFO "mode (low/high) = (%x:%x)\n",
+							tmcc.ts_id[lp].low_mode, tmcc.ts_id[lp].high_mode);
+					printk(KERN_INFO "slot (low/high) = (%x:%x)\n",
+							tmcc.ts_id[lp].low_slot,
+							tmcc.ts_id[lp].high_slot);
 				}
+#endif
 				ts_lock(channel->ptr->regs,
 						channel->ptr->lock,
 						channel->address,
@@ -410,12 +466,12 @@ int		pt1_makering(struct pci_dev *pdev, PT1_DEVICE *dev_conf)
 	for(lp = 0 ; lp < DMA_RING_SIZE ; lp++){
 		ptr = dev_conf->dmaptr[lp];
 		if(lp ==  (DMA_RING_SIZE - 1)){
-			addr = (__u32)dev_conf->dmaptr[0];
+			addr = (__u32)dev_conf->ring_dma[0];
 		}else{
-			addr = (__u32)dev_conf->dmaptr[(lp + 1)];
+			addr = (__u32)dev_conf->ring_dma[(lp + 1)];
 		}
 		addr >>= 12 ;
-		memcpy(ptr, &addr, sizeof(int));
+		memcpy(ptr, &addr, sizeof(__u32));
 		ptr += 1 ;
 
 		dmactl = &dev_conf->dmactl[lp];
@@ -428,7 +484,7 @@ int		pt1_makering(struct pci_dev *pdev, PT1_DEVICE *dev_conf)
 			dmactl->data[lp2] = dmaptr ;
 			// DMAǡꥢ
 			dmaptr[(DMA_SIZE / sizeof(__u32)) - 2] = 0 ;
-			addr = (int)dmaptr;
+			addr = (__u32)dmactl->ring_dma[lp2];
 			addr >>= 12 ;
 			memcpy(ptr, &addr, sizeof(__u32));
 			ptr += 1 ;
@@ -478,22 +534,46 @@ static int __devinit pt1_pci_init_one (struct pci_dev *pdev,
 {
 	int			rc ;
 	int			lp ;
+	int			minor ;
+	u16			cmd ;
 	PT1_DEVICE	*dev_conf ;
 	PT1_CHANNEL	*channel ;
 
 	rc = pci_enable_device(pdev);
 	if (rc)
 		return rc;
+	rc = pci_set_dma_mask(pdev, DMA_32BIT_MASK);
+	if (rc) {
+		printk(KERN_ERR "PT1:DMA MASK ERROR");
+		return rc;
+	}
+
+	pci_read_config_word(pdev, PCI_COMMAND, &cmd);
+	if (!(cmd & PCI_COMMAND_MASTER)) {
+		printk(KERN_INFO "Attempting to enable Bus Mastering\n");
+		pci_set_master(pdev);
+		pci_read_config_word(pdev, PCI_COMMAND, &cmd);
+		if (!(cmd & PCI_COMMAND_MASTER)) {
+			printk(KERN_ERR "Bus Mastering is not enabled\n");
+			return -EIO;
+		}
+	}
+	printk(KERN_INFO "Bus Mastering Enabled.\n");
 
 	dev_conf = kzalloc(sizeof(PT1_DEVICE), GFP_KERNEL);
 	if(!dev_conf){
 		printk(KERN_ERR "PT1:out of memory !");
 		return -ENOMEM ;
 	}
-	device = dev_conf ;
 	// PCIɥ쥹ޥåפ
 	dev_conf->mmio_start = pci_resource_start(pdev, 0);
 	dev_conf->mmio_len = pci_resource_len(pdev, 0);
+	rc = request_mem_region(dev_conf->mmio_start, dev_conf->mmio_len, DEV_NAME);
+	if (!rc) {
+		printk(KERN_ERR "PT1: cannot request iomem  (0x%llx).\n", (unsigned long long) dev_conf->mmio_start);
+		goto out_err_regbase;
+	}
+
 	dev_conf->regs = ioremap(dev_conf->mmio_start, dev_conf->mmio_len);
 	if (!dev_conf->regs){
 		printk(KERN_ERR "pt1: Can't remap register area.\n");
@@ -536,9 +616,18 @@ static int __devinit pt1_pci_init_one (struct pci_dev *pdev,
 	// 
 	init_waitqueue_head(&dev_conf->dma_wait_q);
 
+	minor = MINOR(dev_conf->dev) ;
+	dev_conf->base_minor = minor ;
+	for(lp = 0 ; lp < MAX_PCI_DEVICE ; lp++){
+		if(device[lp] == NULL){
+			device[lp] = dev_conf ;
+			printk(KERN_INFO "Alloc[%d]\n", lp);
+			break ;
+		}
+	}
 	for(lp = 0 ; lp < MAX_CHANNEL ; lp++){
 		cdev_init(&dev_conf->cdev[lp], &pt1_fops);
-		cdev_add(&dev_conf->cdev[lp], MKDEV(MAJOR(dev_conf->dev), lp), 1);
+		cdev_add(&dev_conf->cdev[lp], MKDEV(MAJOR(dev_conf->dev), (MINOR(dev_conf->dev) + lp)), 1);
 		channel = kzalloc(sizeof(PT1_CHANNEL), GFP_KERNEL);
 		if(!channel){
 			printk(KERN_ERR "PT1:out of memory !");
@@ -549,6 +638,9 @@ static int __devinit pt1_pci_init_one (struct pci_dev *pdev,
 		spin_lock_init(&channel->lock);
 		// Ԥ֤
 		channel->req_dma = FALSE ;
+		// ޥʡֹ
+		channel->minor = MINOR(dev_conf->dev) + lp ;
+		printk(KERN_INFO "Minor[%d]\n", channel->minor);
 		// оݤI2CǥХ
 		channel->address = i2c_address[lp] ;
 		channel->type = channeltype[lp] ;
@@ -602,6 +694,7 @@ out_err_fpga:
 	writel(0xb0b0000, dev_conf->regs);
 	writel(0, dev_conf->regs + 4);
 	iounmap(dev_conf->regs);
+	release_mem_region(dev_conf->mmio_start, dev_conf->mmio_len);
 	kfree(dev_conf);
 out_err_regbase:
 	return -EIO;
@@ -642,6 +735,7 @@ static void __devexit pt1_pci_remove_one(struct pci_dev *pdev)
 		writel(0xb0b0000, dev_conf->regs);
 		writel(0, dev_conf->regs + 4);
 		settuner_reset(dev_conf->regs, LNB_OFF, TUNER_POWER_OFF);
+		release_mem_region(dev_conf->mmio_start, dev_conf->mmio_len);
 		iounmap(dev_conf->regs);
 		kfree(dev_conf);
 	}
